# RateMyUWPlan API Documentation
The RateMyUWPlan API handles the storage, creation and handling of reviews for the RateMyUWPlan browser extension.

## Get all reviews for a class
**Request format:** /reviews/:classname

**Request type:** GET

**Returned Data Format:** JSON

**Description:** Returns data for all reviews for a given class

**Example Request:** /reviews/CSE 403

**Example Response:**
```json
[
  {
    "review_id": 1,
    "difficulty_rating": 3,
    "time_rating": 5,
    "fun_rating": 3,
    "overall_rating": 4,
    "review_text": "Very fun class would recommend.",
    "user": "realjlee",
    "class":"CSE 403",
    "is_anon": 0,
    "username":" realjlee",
    "displayname": "Junha Lee"
  }
]
```

**Error Handling:**
N/A

## Get all reviews for a user
**Request format:** /reviews/user/:username

**Request type:** GET

**Returned Data Format:** JSON

**Description:** Returns data for all reviews for a given user

**Example Request:** /reviews/user/realjlee

**Example Response:**
```json
[
  {
    "review_id": 1,
    "difficulty_rating": 3,
    "time_rating": 5,
    "fun_rating": 3,
    "overall_rating": 4,
    "review_text": "Very fun class would recommend.",
    "user": "realjlee",
    "class":"CSE 403",
    "is_anon": 0,
    "username":" realjlee",
    "displayname": "Junha Lee"
  }
]
```

**Error Handling:**
N/A

## Get average ratings for a class
**Request format:** /reviews/:classname/ratings

**Request type:** GET

**Returned Data Format:** JSON

**Description:** Returns average ratings for a given class

**Example Request:** /reviews/ART 190/ratings

**Example Response:**
```json
[
  {
    "avg_difficulty": 4,
    "avg_time": 4,
    "avg_fun": 2.5,
    "avg_overall": 2.5
  }
]
```

**Error Handling:**
N/A

## Post a review
**Request format:** /reviews/post

**Request type:** POST

**Returned Data Format:** Plain Text

**Description:** Adds a review to a class with given ratings and description

**Example Request:** /reviews/post with POST parameters:
- `difficulty-rating=4`
- `time-rating=2`
- `fun-rating=4`
- `overall-rating=4`
- `review-text=I like this class a lot.`
- `class-name=CSE 457`
- `username=realjlee`
- `is-anon=on`

(Note, is-anon is "on" when review left should be anonymous, undefined when not anonymous, due to how FormData handles checkboxes)

**Example Response:**
```Review successfully added```

**Error Handling:**
- Possible 400 (invalid request) error (plain text)
  - If any POST parameters are missing an error is returned with `One or more parameters missing.`
  - If any rating parameters aren't integers between 1-5 an error is returned with `One or more ratings are invalid. (Must be integers between 1-5)`
  - If is-anon post parameter isn't 0 or 1, an error is returned with `is-anon POST parameter must be 0 or 1.`
  - If username doesn't exist in database an error is returned with `Invalid username given; user does not exist`
  - If the user has already posted a review for the class an error is returned with `User has already left a review for this class: [reviewID]`


## Delete a review
**Request format:** /reviews/delete

**Request type:** POST

**Returned Data Format:** Plain Text

**Description:** Deletes a review

**Example Request:** /reviews/delete with POST parameters:
- `review-id=1`

**Example Response:**
```Review successfully deleted.```

**Error Handling:**
- Possible 400 (invalid request) error (plain text)
  - If any POST parameters are missing an error is returned with `One or more parameters missing.`
  - If review id being deleted doesn't exist an error is returned with `Invalid review to delete.`

## Edit a review
**Request format:** /reviews/update

**Request type:** POST

**Returned Data Format:** Plain Text

**Description:** Updates a review's rating values, body text, and anonymity

**Example Request:** /reviews/post with POST parameters:
- `difficulty-rating=5`
- `time-rating=5`
- `fun-rating=1`
- `overall-rating=1`
- `review-text=I changed my mind I hate this class.`
- `is-anon=on`
- `review-id=3`

(Note, is-anon is "on" when review left should be anonymous, undefined when not anonymous, due to how FormData handles checkboxes)

**Example Response:**
```Review successfully updated```

**Error Handling:**
- Possible 400 (invalid request) error (plain text)
  - If any POST parameters are missing an error is returned with `One or more parameters missing.`
  - If any rating parameters aren't integers between 1-5 an error is returned with `One or more ratings are invalid. (Must be integers between 1-5)`
  - If is-anon post parameter isn't 0 or 1, an error is returned with `is-anon POST parameter must be 0 or 1.`
  - If the review being updated doesn't exist an error is returned with `Invalid review to update.`

## Log in to RateMyUWPlan
**Request format:** /users/login

**Request type:** POST

**Returned Data Format:** Plain Text

**Description:** Authenticates a username and logs them in

**Example Request:** /users/login with POST parameters:
- `username=realjlee`

**Example Response:**
```Login success.```

**Error Handling:**
- Possible 400 (invalid request) error (plain text)
  - If any POST parameters are missing an error is returned with `One or more parameters missing.`
  - If username doesn't exist in database an error is returned with `Invalid login creditials`

## Create a new user
**Request format:** /users/create

**Request type:** POST

**Returned Data Format:** Plain Text

**Description:** Adds a new user to the system

**Example Request:** /users/create with POST parameters:
- `username=realjlee`
- `display-name=Junha Lee`

**Example Response:**
```User successfully added```

**Error Handling:**
- Possible 400 (invalid request) error (plain text)
  - If any POST parameters are missing an error is returned with `One or more parameters missing.`
  - If username already exists in database an error is returned with `Username is already taken.`