# RateMyUWPlan

## Description
RateMyUWPlan is a browser extension designed for use with UW MyPlan. It aims to help people with the process of signing up for classes by adding an integrated button to the MyPlan page for a UW class, that when clicked, opens a list of student-provided ratings and reviews concerning the class. Students can add their own reviews via the extension dropdown from the top right of the browser.

## Installation
Click [here](https://chrome.google.com/webstore/detail/ratemyuwplan/ijgopmdkjpakkgpfbinmkgoblkdpigfa?hl=en&authuser=0) for the latest
public RateMyUWPlan release!

## Authors and acknowledgment
Junha, Clarence, Ali, Kenneth, Bella, Brenda

## Project structure
Main folder contains `app.js`, `package.json`, `reviews.db`, all backend stuff.

`public` directory contains frontend files.
`index.html`, `index.js`, `login.html`, `login.js` pertain to the extension popup in the top right
`content.js` is for DOM manipulation, adding the "reviews" button.
`background.js` is a background script for chrome extensions constantly running the background that handles functionality
that `content.js` and scripts in the extension popup can't handle.

## Operational use cases
- Lookup reviews
- Write a review
- Write a review as an anonymous user
- Edit a review by overwriting
- Delete a review
- Login/Logout
- Create new user

## Version control
When starting work on a new feature, please create a new branch to work on before merging it back to the main branch.

## Bug tracking
Any bugs that have been found should be added to the gitlab issues. If important, please notify the other team members
on slack!

## Build system
The build system for the backend server uses `node.js` and `npm`. The server can be started by running either `node app.js`
or `npm start` in the root directory for this project. Building the frontend is done by loading the unpacked extension in
the `public` directory into google chrome, more details under the Setup section of this README.

## Setup
### Locally running backend server
NOTE: The backend is currently hosted on attu, so this step is only necessary if you plan on making any changes to the backend
`app.js`.

First, in `login.js`, `main.js`, and `background.js` you must set the LOCAL_DEBUG global constant to true (Note: will streamline
this process later).

After cloning or downloading the project files, there are some other things we need.
First you must install `node.js` on your machine (nodejs.org).
Must run `npm install` to add the required project dependencies. We are gitignoring the node_modules folder so this is necessary.

In order to view the data in `reviews.db` for debugging/other purposes, download the [SQLite DB Browser](https://sqlitebrowser.org/)
or get the [SQLite3 Editor Extension](https://marketplace.visualstudio.com/items?itemName=yy0931.vscode-sqlite3-editor) for vscode.

To start the locally hosted backend, go to an appropriate CLI and execute `node app.js` or `npm start` inside the project directory.
The locally hosted backend must be running in order for all the extension features to work properly.

## Testing and CI
Tests will automatically run in CI everytime code is committed and pushed to the gitlab repository, however currently the only
registered runner is Junha's laptop, so the CI may error due to timeout.
To run test scripts locally, execute `npm test`.

### Extension Installation Guide
In the current development cycle we are only supporting/testing on chrome. While the extension should theoretically work on
any chromium-based browser, these browsers have not been tested. Use at your own risk.

To install the extension in chrome, first go to `chrome://extensions`. Toggle the button in the top right to turn on developer mode,
then click the "Load unpacked" button. Navigate to the `/public` directory and select it. The extension is now loaded! You should see
the extension icon in the top right of your browser, and course pages in MyPlan should now have an additional button added to the page.