const LOCAL_DEBUG = false;
const url = (LOCAL_DEBUG) ? "http://localhost:2903" : "http://attu7.cs.washington.edu:2903";

let currUser = null;

const keepAlive = () => setInterval(chrome.runtime.getPlatformInfo, 20e3);
chrome.runtime.onStartup.addListener(keepAlive);
keepAlive();

chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.type === "getdata") {
        fetch(url + "/reviews/" + message.class)
            .then(statusCheck)
            .then(resp => resp.json())
            .then((resp) => {
                console.log(resp);
                sendResponse(resp)
            })
            .catch(console.error);
        return true;
    }
});

// Listens for message from login.js, sets the currUser to the user that just logged in
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.type === "login") {
        currUser = message.user;
        // console.log("Message recieved: setting curr user to " + currUser);
        return true;
    }
});

// Listens for message from login.js that sets the curr user to be logged out
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.type === "logout") {
        console.log("logging out user");
        currUser = null;
        return true;
    }
});

// Listens for message that checks which user is currently logged in
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.type === "getCurrUser") {
        // console.log("Message recieved: curr user is " + currUser);
        sendResponse(currUser);
        return true;
    }
});

chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.type === "deleteReview") {
        console.log("recieved delete message: " + message.id);
        let formData = new FormData();
        formData.append("review-id", message.id);

        fetch(url + "/reviews/delete", {method: 'POST', body: formData})
            .then(statusCheck)
            .then(resp => resp.text())
            .then((resp) => {
                console.log(resp);
            })
            .catch(console.error);
        return true;
    }
});


/**
   * Asynchronous function that checks whether a fetch call was successful.
   * @param {object} response - JSON object representing the response
   * @returns {object} - The same response, if the status check was successful.
   */
async function statusCheck(response) {
    if (!response.ok) {
        throw new Error(await response.text());
    }
    return response;
}