//During the test the env variable is set to test
process.env.NODE_ENV = 'test';

//Require the dev-dependencies
let chai = require('chai');
let chaiHTTP = require('chai-http');
let server = require("../app");
let should = chai.should();

chai.use(chaiHTTP);

describe("Basic GET tests", () => {
    describe("/GET test (Sanity check)", () => {
        it("it should GET 'Hello world'", (done) => {
            chai.request(server)
                .get("/test")
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.text;
                    res.text.should.equal("Hello world.");
                    done();
                });
        });
    });
    describe("/GET reviews/:classid", () => {
        it("it should GET reviews for a class", (done) => {
            chai.request(server)
                .get("/reviews/CSE 344")
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    done();
                });
        });
    });
    describe("/GET reviews/:classid/ratings", () => {
        it("it should GET average ratings for a class", (done) => {
            chai.request(server)
                .get("/reviews/CSE 344/ratings")
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    done();
                });
        });
    });
    describe("/GET reviews/user/:username", () => {
        it("it should GET reviews for a user", (done) => {
            chai.request(server)
                .get("/reviews/user/realjlee")
                .end((err, res) => {
                    res.should.have.status(200);
                    res.should.be.json;
                    done();
                });
        });
    });
});

describe("POST error handling tests", () => {
    describe("/POST reviews/post", () => {
        it("it should fail when difficulty-rating is missing", (done) => {
            let review = {
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when time-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when fun-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when overall-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when review-text is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when class-name is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when username is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when a rating is less than 1", (done) => {
            let review = {
                "difficulty-rating": "0",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating is greater than 5", (done) => {
            let review = {
                "difficulty-rating": "6",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating is nonsense", (done) => {
            let review = {
                "difficulty-rating": "alsdfkjasdlkjf",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating contains a valid rating but isn't one (edge case)", (done) => {
            let review = {
                "difficulty-rating": "3aksdf",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when the username doesn't exist in the database", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 121",
                "username": "asdfsdf",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid username given; user does not exist");
                    done();
                });
        });
        it("it should fail when the user has already left a review for the class", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "class-name": "CSE 403",
                "username": "realjlee",
                "is-anon": "on"
            };
            chai.request(server)
                .post("/reviews/post")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.contain("User has already left a review for this class:");
                    done();
                });
        });
    });
    describe("/POST reviews/delete", () => {
        it("it should fail when review-id is not given", (done) => {
            let body = {
            };
            chai.request(server)
                .post("/reviews/delete")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when review-id doesn't exist", (done) => {
            let body = {
                "review-id": "123455"
            };
            chai.request(server)
                .post("/reviews/delete")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid review to delete.");
                    done();
                });
        });
        it("it should fail when review-id is garbage", (done) => {
            let body = {
                "review-id": "askdclaskdjflaskjf"
            };
            chai.request(server)
                .post("/reviews/delete")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid review to delete.");
                    done();
                });
        });
    });
    describe("/POST reviews/update", () => {
        it("it should fail when difficulty-rating is missing", (done) => {
            let review = {
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when time-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when fun-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when overall-rating is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when review-text is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when review-id is missing", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when a rating is less than 1", (done) => {
            let review = {
                "difficulty-rating": "0",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating is greater than 5", (done) => {
            let review = {
                "difficulty-rating": "6",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating is nonsense", (done) => {
            let review = {
                "difficulty-rating": "alskdjflaskjdf",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when a rating contains a valid rating but isn't one (edge case)", (done) => {
            let review = {
                "difficulty-rating": "3alskdcj",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "1"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more ratings are invalid. (Must be integers between 1-5)");
                    done();
                });
        });
        it("it should fail when review-id doesn't exist", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "99999999"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid review to update.");
                    done();
                });
        });
        it("it should fail when review-id is garbage", (done) => {
            let review = {
                "difficulty-rating": "3",
                "time-rating": "3",
                "fun-rating": "3",
                "overall-rating": "3",
                "review-text": "test review",
                "is-anon": "on",
                "review-id": "alskdcnlskd"
            };
            chai.request(server)
                .post("/reviews/update")
                .send(review)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid review to update.");
                    done();
                });
        });
    });
    describe("/POST users/login", () => {
        it("it should fail when username is missing", (done) => {
            let body = {
            };
            chai.request(server)
                .post("/users/login")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when username is invalid", (done) => {
            let body = {
                "username": "laskdjlkasjboib3o2pi"
            };
            chai.request(server)
                .post("/users/login")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Invalid login creditials");
                    done();
                });
        });
    });
    describe("/POST users/create", () => {
        it("it should fail when username is missing", (done) => {
            let body = {
                "display-name": "Junha Lee"
            };
            chai.request(server)
                .post("/users/create")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when display name is missing", (done) => {
            let body = {
                "username": "testfailuser"
            };
            chai.request(server)
                .post("/users/create")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("One or more parameters missing.");
                    done();
                });
        });
        it("it should fail when username is taken", (done) => {
            let body = {
                "username": "realjlee",
                "display-name": "Junha Lee"
            };
            chai.request(server)
                .post("/users/create")
                .send(body)
                .end((err, res) => {
                    res.should.have.status(400);
                    res.should.be.text;
                    res.text.should.equal("Username is already taken.");
                    done();
                });
        });
    });
});